'''OpenGL extension ARB.vertex_shader

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_vertex_shader'
_DEPRECATED = False
GL_VERTEX_SHADER_ARB = constant.Constant( 'GL_VERTEX_SHADER_ARB', 0x8B31 )
GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB = constant.Constant( 'GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB', 0x8B4A )
glget.addGLGetConstant( GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB, (1,) )
GL_MAX_VARYING_FLOATS_ARB = constant.Constant( 'GL_MAX_VARYING_FLOATS_ARB', 0x8B4B )
glget.addGLGetConstant( GL_MAX_VARYING_FLOATS_ARB, (1,) )
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB = constant.Constant( 'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB', 0x8B4C )
glget.addGLGetConstant( GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB, (1,) )
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB = constant.Constant( 'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB', 0x8B4D )
glget.addGLGetConstant( GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB, (1,) )
GL_OBJECT_ACTIVE_ATTRIBUTES_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_ATTRIBUTES_ARB', 0x8B89 )
GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB = constant.Constant( 'GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB', 0x8B8A )
glBindAttribLocationARB = platform.createExtensionFunction( 
'glBindAttribLocationARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLuint,arrays.GLcharARBArray,),
doc='glBindAttribLocationARB(GLhandleARB(programObj), GLuint(index), GLcharARBArray(name)) -> None',
argNames=('programObj','index','name',),
deprecated=_DEPRECATED,
)

glGetActiveAttribARB = platform.createExtensionFunction( 
'glGetActiveAttribARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLhandleARB,constants.GLuint,constants.GLsizei,arrays.GLsizeiArray,arrays.GLintArray,arrays.GLuintArray,arrays.GLcharARBArray,),
doc='glGetActiveAttribARB(GLhandleARB(programObj), GLuint(index), GLsizei(maxLength), GLsizeiArray(length), GLintArray(size), GLuintArray(type), GLcharARBArray(name)) -> None',
argNames=('programObj','index','maxLength','length','size','type','name',),
deprecated=_DEPRECATED,
)

glGetAttribLocationARB = platform.createExtensionFunction( 
'glGetAttribLocationARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLint, 
argTypes=(constants.GLhandleARB,arrays.GLcharARBArray,),
doc='glGetAttribLocationARB(GLhandleARB(programObj), GLcharARBArray(name)) -> constants.GLint',
argNames=('programObj','name',),
deprecated=_DEPRECATED,
)


def glInitVertexShaderARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.shadow

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_shadow'
_DEPRECATED = False
GL_TEXTURE_COMPARE_MODE_ARB = constant.Constant( 'GL_TEXTURE_COMPARE_MODE_ARB', 0x884C )
GL_TEXTURE_COMPARE_FUNC_ARB = constant.Constant( 'GL_TEXTURE_COMPARE_FUNC_ARB', 0x884D )
GL_COMPARE_R_TO_TEXTURE_ARB = constant.Constant( 'GL_COMPARE_R_TO_TEXTURE_ARB', 0x884E )


def glInitShadowARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.pixel_transform

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_pixel_transform'
_DEPRECATED = False
GL_PIXEL_TRANSFORM_2D_EXT = constant.Constant( 'GL_PIXEL_TRANSFORM_2D_EXT', 0x8330 )
GL_PIXEL_MAG_FILTER_EXT = constant.Constant( 'GL_PIXEL_MAG_FILTER_EXT', 0x8331 )
GL_PIXEL_MIN_FILTER_EXT = constant.Constant( 'GL_PIXEL_MIN_FILTER_EXT', 0x8332 )
GL_PIXEL_CUBIC_WEIGHT_EXT = constant.Constant( 'GL_PIXEL_CUBIC_WEIGHT_EXT', 0x8333 )
GL_CUBIC_EXT = constant.Constant( 'GL_CUBIC_EXT', 0x8334 )
GL_AVERAGE_EXT = constant.Constant( 'GL_AVERAGE_EXT', 0x8335 )
GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = constant.Constant( 'GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT', 0x8336 )
glget.addGLGetConstant( GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT, (1,) )
GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = constant.Constant( 'GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT', 0x8337 )
glget.addGLGetConstant( GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT, (1,) )
GL_PIXEL_TRANSFORM_2D_MATRIX_EXT = constant.Constant( 'GL_PIXEL_TRANSFORM_2D_MATRIX_EXT', 0x8338 )
glPixelTransformParameteriEXT = platform.createExtensionFunction( 
'glPixelTransformParameteriEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,),
doc='glPixelTransformParameteriEXT(GLenum(target), GLenum(pname), GLint(param)) -> None',
argNames=('target','pname','param',),
deprecated=_DEPRECATED,
)

glPixelTransformParameterfEXT = platform.createExtensionFunction( 
'glPixelTransformParameterfEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLfloat,),
doc='glPixelTransformParameterfEXT(GLenum(target), GLenum(pname), GLfloat(param)) -> None',
argNames=('target','pname','param',),
deprecated=_DEPRECATED,
)

glPixelTransformParameterivEXT = platform.createExtensionFunction( 
'glPixelTransformParameterivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glPixelTransformParameterivEXT(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glPixelTransformParameterfvEXT = platform.createExtensionFunction( 
'glPixelTransformParameterfvEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glPixelTransformParameterfvEXT(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)


def glInitPixelTransformEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension HP.occlusion_test

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_HP_occlusion_test'
_DEPRECATED = False
GL_OCCLUSION_TEST_HP = constant.Constant( 'GL_OCCLUSION_TEST_HP', 0x8165 )
glget.addGLGetConstant( GL_OCCLUSION_TEST_HP, (1,) )
GL_OCCLUSION_TEST_RESULT_HP = constant.Constant( 'GL_OCCLUSION_TEST_RESULT_HP', 0x8166 )
glget.addGLGetConstant( GL_OCCLUSION_TEST_RESULT_HP, (1,) )


def glInitOcclusionTestHP():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.vertex_array

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_vertex_array'
_DEPRECATED = False
GL_VERTEX_ARRAY_EXT = constant.Constant( 'GL_VERTEX_ARRAY_EXT', 0x8074 )
glget.addGLGetConstant( GL_VERTEX_ARRAY_EXT, (1,) )
GL_NORMAL_ARRAY_EXT = constant.Constant( 'GL_NORMAL_ARRAY_EXT', 0x8075 )
glget.addGLGetConstant( GL_NORMAL_ARRAY_EXT, (1,) )
GL_COLOR_ARRAY_EXT = constant.Constant( 'GL_COLOR_ARRAY_EXT', 0x8076 )
GL_INDEX_ARRAY_EXT = constant.Constant( 'GL_INDEX_ARRAY_EXT', 0x8077 )
glget.addGLGetConstant( GL_INDEX_ARRAY_EXT, (1,) )
GL_TEXTURE_COORD_ARRAY_EXT = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_EXT', 0x8078 )
glget.addGLGetConstant( GL_TEXTURE_COORD_ARRAY_EXT, (1,) )
GL_EDGE_FLAG_ARRAY_EXT = constant.Constant( 'GL_EDGE_FLAG_ARRAY_EXT', 0x8079 )
glget.addGLGetConstant( GL_EDGE_FLAG_ARRAY_EXT, (1,) )
GL_VERTEX_ARRAY_SIZE_EXT = constant.Constant( 'GL_VERTEX_ARRAY_SIZE_EXT', 0x807A )
glget.addGLGetConstant( GL_VERTEX_ARRAY_SIZE_EXT, (1,) )
GL_VERTEX_ARRAY_TYPE_EXT = constant.Constant( 'GL_VERTEX_ARRAY_TYPE_EXT', 0x807B )
glget.addGLGetConstant( GL_VERTEX_ARRAY_TYPE_EXT, (1,) )
GL_VERTEX_ARRAY_STRIDE_EXT = constant.Constant( 'GL_VERTEX_ARRAY_STRIDE_EXT', 0x807C )
glget.addGLGetConstant( GL_VERTEX_ARRAY_STRIDE_EXT, (1,) )
GL_VERTEX_ARRAY_COUNT_EXT = constant.Constant( 'GL_VERTEX_ARRAY_COUNT_EXT', 0x807D )
glget.addGLGetConstant( GL_VERTEX_ARRAY_COUNT_EXT, (1,) )
GL_NORMAL_ARRAY_TYPE_EXT = constant.Constant( 'GL_NORMAL_ARRAY_TYPE_EXT', 0x807E )
glget.addGLGetConstant( GL_NORMAL_ARRAY_TYPE_EXT, (1,) )
GL_NORMAL_ARRAY_STRIDE_EXT = constant.Constant( 'GL_NORMAL_ARRAY_STRIDE_EXT', 0x807F )
glget.addGLGetConstant( GL_NORMAL_ARRAY_STRIDE_EXT, (1,) )
GL_NORMAL_ARRAY_COUNT_EXT = constant.Constant( 'GL_NORMAL_ARRAY_COUNT_EXT', 0x8080 )
glget.addGLGetConstant( GL_NORMAL_ARRAY_COUNT_EXT, (1,) )
GL_COLOR_ARRAY_SIZE_EXT = constant.Constant( 'GL_COLOR_ARRAY_SIZE_EXT', 0x8081 )
glget.addGLGetConstant( GL_COLOR_ARRAY_SIZE_EXT, (1,) )
GL_COLOR_ARRAY_TYPE_EXT = constant.Constant( 'GL_COLOR_ARRAY_TYPE_EXT', 0x8082 )
glget.addGLGetConstant( GL_COLOR_ARRAY_TYPE_EXT, (1,) )
GL_COLOR_ARRAY_STRIDE_EXT = constant.Constant( 'GL_COLOR_ARRAY_STRIDE_EXT', 0x8083 )
glget.addGLGetConstant( GL_COLOR_ARRAY_STRIDE_EXT, (1,) )
GL_COLOR_ARRAY_COUNT_EXT = constant.Constant( 'GL_COLOR_ARRAY_COUNT_EXT', 0x8084 )
glget.addGLGetConstant( GL_COLOR_ARRAY_COUNT_EXT, (1,) )
GL_INDEX_ARRAY_TYPE_EXT = constant.Constant( 'GL_INDEX_ARRAY_TYPE_EXT', 0x8085 )
glget.addGLGetConstant( GL_INDEX_ARRAY_TYPE_EXT, (1,) )
GL_INDEX_ARRAY_STRIDE_EXT = constant.Constant( 'GL_INDEX_ARRAY_STRIDE_EXT', 0x8086 )
glget.addGLGetConstant( GL_INDEX_ARRAY_STRIDE_EXT, (1,) )
GL_INDEX_ARRAY_COUNT_EXT = constant.Constant( 'GL_INDEX_ARRAY_COUNT_EXT', 0x8087 )
glget.addGLGetConstant( GL_INDEX_ARRAY_COUNT_EXT, (1,) )
GL_TEXTURE_COORD_ARRAY_SIZE_EXT = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_SIZE_EXT', 0x8088 )
glget.addGLGetConstant( GL_TEXTURE_COORD_ARRAY_SIZE_EXT, (1,) )
GL_TEXTURE_COORD_ARRAY_TYPE_EXT = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_TYPE_EXT', 0x8089 )
glget.addGLGetConstant( GL_TEXTURE_COORD_ARRAY_TYPE_EXT, (1,) )
GL_TEXTURE_COORD_ARRAY_STRIDE_EXT = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_STRIDE_EXT', 0x808A )
glget.addGLGetConstant( GL_TEXTURE_COORD_ARRAY_STRIDE_EXT, (1,) )
GL_TEXTURE_COORD_ARRAY_COUNT_EXT = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_COUNT_EXT', 0x808B )
glget.addGLGetConstant( GL_TEXTURE_COORD_ARRAY_COUNT_EXT, (1,) )
GL_EDGE_FLAG_ARRAY_STRIDE_EXT = constant.Constant( 'GL_EDGE_FLAG_ARRAY_STRIDE_EXT', 0x808C )
glget.addGLGetConstant( GL_EDGE_FLAG_ARRAY_STRIDE_EXT, (1,) )
GL_EDGE_FLAG_ARRAY_COUNT_EXT = constant.Constant( 'GL_EDGE_FLAG_ARRAY_COUNT_EXT', 0x808D )
glget.addGLGetConstant( GL_EDGE_FLAG_ARRAY_COUNT_EXT, (1,) )
GL_VERTEX_ARRAY_POINTER_EXT = constant.Constant( 'GL_VERTEX_ARRAY_POINTER_EXT', 0x808E )
GL_NORMAL_ARRAY_POINTER_EXT = constant.Constant( 'GL_NORMAL_ARRAY_POINTER_EXT', 0x808F )
GL_COLOR_ARRAY_POINTER_EXT = constant.Constant( 'GL_COLOR_ARRAY_POINTER_EXT', 0x8090 )
GL_INDEX_ARRAY_POINTER_EXT = constant.Constant( 'GL_INDEX_ARRAY_POINTER_EXT', 0x8091 )
GL_TEXTURE_COORD_ARRAY_POINTER_EXT = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_POINTER_EXT', 0x8092 )
GL_EDGE_FLAG_ARRAY_POINTER_EXT = constant.Constant( 'GL_EDGE_FLAG_ARRAY_POINTER_EXT', 0x8093 )
glArrayElementEXT = platform.createExtensionFunction( 
'glArrayElementEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,),
doc='glArrayElementEXT(GLint(i)) -> None',
argNames=('i',),
deprecated=_DEPRECATED,
)

glColorPointerEXT = platform.createExtensionFunction( 
'glColorPointerEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLenum,constants.GLsizei,constants.GLsizei,ctypes.c_void_p,),
doc='glColorPointerEXT(GLint(size), GLenum(type), GLsizei(stride), GLsizei(count), c_void_p(pointer)) -> None',
argNames=('size','type','stride','count','pointer',),
deprecated=_DEPRECATED,
)

glDrawArraysEXT = platform.createExtensionFunction( 
'glDrawArraysEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,constants.GLsizei,),
doc='glDrawArraysEXT(GLenum(mode), GLint(first), GLsizei(count)) -> None',
argNames=('mode','first','count',),
deprecated=_DEPRECATED,
)

glEdgeFlagPointerEXT = platform.createExtensionFunction( 
'glEdgeFlagPointerEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLsizei,constants.GLsizei,arrays.GLbooleanArray,),
doc='glEdgeFlagPointerEXT(GLsizei(stride), GLsizei(count), GLbooleanArray(pointer)) -> None',
argNames=('stride','count','pointer',),
deprecated=_DEPRECATED,
)

glGetPointervEXT = platform.createExtensionFunction( 
'glGetPointervEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,ctypes.POINTER(ctypes.c_void_p),),
doc='glGetPointervEXT(GLenum(pname), POINTER(ctypes.c_void_p)(params)) -> None',
argNames=('pname','params',),
deprecated=_DEPRECATED,
)

glIndexPointerEXT = platform.createExtensionFunction( 
'glIndexPointerEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,constants.GLsizei,ctypes.c_void_p,),
doc='glIndexPointerEXT(GLenum(type), GLsizei(stride), GLsizei(count), c_void_p(pointer)) -> None',
argNames=('type','stride','count','pointer',),
deprecated=_DEPRECATED,
)

glNormalPointerEXT = platform.createExtensionFunction( 
'glNormalPointerEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,constants.GLsizei,ctypes.c_void_p,),
doc='glNormalPointerEXT(GLenum(type), GLsizei(stride), GLsizei(count), c_void_p(pointer)) -> None',
argNames=('type','stride','count','pointer',),
deprecated=_DEPRECATED,
)

glTexCoordPointerEXT = platform.createExtensionFunction( 
'glTexCoordPointerEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLenum,constants.GLsizei,constants.GLsizei,ctypes.c_void_p,),
doc='glTexCoordPointerEXT(GLint(size), GLenum(type), GLsizei(stride), GLsizei(count), c_void_p(pointer)) -> None',
argNames=('size','type','stride','count','pointer',),
deprecated=_DEPRECATED,
)

glVertexPointerEXT = platform.createExtensionFunction( 
'glVertexPointerEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLenum,constants.GLsizei,constants.GLsizei,ctypes.c_void_p,),
doc='glVertexPointerEXT(GLint(size), GLenum(type), GLsizei(stride), GLsizei(count), c_void_p(pointer)) -> None',
argNames=('size','type','stride','count','pointer',),
deprecated=_DEPRECATED,
)


def glInitVertexArrayEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.shading_language_100

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_shading_language_100'
_DEPRECATED = False
GL_SHADING_LANGUAGE_VERSION_ARB = constant.Constant( 'GL_SHADING_LANGUAGE_VERSION_ARB', 0x8B8C )


def glInitShadingLanguage100ARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.depth_bounds_test

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_depth_bounds_test'
_DEPRECATED = False
GL_DEPTH_BOUNDS_TEST_EXT = constant.Constant( 'GL_DEPTH_BOUNDS_TEST_EXT', 0x8890 )
glget.addGLGetConstant( GL_DEPTH_BOUNDS_TEST_EXT, (1,) )
GL_DEPTH_BOUNDS_EXT = constant.Constant( 'GL_DEPTH_BOUNDS_EXT', 0x8891 )
glget.addGLGetConstant( GL_DEPTH_BOUNDS_EXT, (1,) )
glDepthBoundsEXT = platform.createExtensionFunction( 
'glDepthBoundsEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLclampd,constants.GLclampd,),
doc='glDepthBoundsEXT(GLclampd(zmin), GLclampd(zmax)) -> None',
argNames=('zmin','zmax',),
deprecated=_DEPRECATED,
)


def glInitDepthBoundsTestEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



#!/usr/bin/env python
'''Hacked version of Pyglet's gengl that produces PyOpenGL-based modules

To use, you have to do an svn co of pyglet to get the "tools" directory,
add the package "wraptypes" to your site-packages and you should be able 
to run this script.

Note: currently the AGL module can only be generated on OS-X, GLX can only
be generated on a GLX platform (e.g. Linux), WGL should generate anywhere
as I'm using Alex's hacked/derived wgl.h as the source for it.

TODO:
    This wrapper is missing a lot of the operations from the original 
    openglgenerator module.  It's only appropriate for generating the 
    platform-specific modules (GLX, WGL, AGL).  Eventually should port 
    the code from the openglgenerator module to support creating the 
    extensions and core GL modules.
'''
# Copyright (c) 2006-2008 Alex Holkner
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
__docformat__ = 'restructuredtext'
__version__ = '$Id: gengl.py,v 1.3 2008/05/02 18:49:57 mcfletch Exp $'

import marshal
import optparse
import os.path
import urllib2
import sys
import textwrap
import re

# monkey-patching the tool to support argument names...
from wraptypes import ctypesparser
class CtypesFunction( ctypesparser.CtypesFunction ):
    def __init__(self, restype, parameters):
        if parameters and parameters[-1] == '...':
             parameters = parameters[:-1]		
        self.argnames = [ self.argname(p.declarator) for p in parameters ]
        super( CtypesFunction, self ).__init__( restype, parameters )
    def argname( self, parameter ):
        if not getattr( parameter, 'pointer', None ):
            return parameter
        else:
            return self.argname( parameter.pointer )
ctypesparser.CtypesFunction = CtypesFunction

from wraptypes import wrap

script_dir = os.path.abspath(os.path.dirname(__file__))

GLEXT_ABI_H = 'http://oss.sgi.com/projects/ogl-sample/ABI/glext.h'
GLEXT_NV_H = 'http://developer.download.nvidia.com/opengl/includes/glext.h'
GLXEXT_ABI_H = 'http://oss.sgi.com/projects/ogl-sample/ABI/glxext.h'
GLXEXT_NV_H = 'http://developer.download.nvidia.com/opengl/includes/glxext.h'
WGLEXT_ABI_H = 'http://oss.sgi.com/projects/ogl-sample/ABI/wglext.h'
WGLEXT_NV_H = 'http://developer.download.nvidia.com/opengl/includes/wglext.h'

AGL_H = '/System/Library/Frameworks/AGL.framework/Headers/agl.h'
GL_H = '/usr/include/GL/gl.h'
GLU_H = '/usr/include/GL/glu.h'
GLX_H = '/usr/include/GL/glx.h'
WGL_H = os.path.join(script_dir, 'wgl.h')

CACHE_FILE = os.path.join(script_dir, '.gengl.cache')
_cache = {}


def load_cache():
    global _cache
    if os.path.exists(CACHE_FILE):
        try:
            _cache = marshal.load(open(CACHE_FILE, 'rb')) or {}
        except:
            pass
    _cache = {}

def save_cache():
    try:
        marshal.dump(_cache, open(CACHE_FILE, 'wb'))
    except:
        pass

def read_url(url):
    if url in _cache:
        return _cache[url]
    if os.path.exists(url):
        data = open(url).read()
    else:
        data = urllib2.urlopen(url).read()
    _cache[url] = data
    save_cache()
    return data

class GLWrapper(wrap.CtypesWrapper):
    requires = None
    requires_prefix = None

    def __init__(self, header, match_re):
        self.header = header
        self.match_re = match_re
        super(GLWrapper, self).__init__()
    def handle_declaration(self, declaration, filename, lineno):
        """Overridden solely to pass in the argument names"""
        t = wrap.get_ctypes_type(declaration.type, declaration.declarator)
        declarator = declaration.declarator
        if declarator is None:
            # XXX TEMPORARY while struct with no typedef not filled in
            return
        while declarator.pointer:
            declarator = declarator.pointer
        name = declarator.identifier
        if declaration.storage == 'typedef':
            self.handle_ctypes_type_definition(
                name, wrap.remove_function_pointer(t), filename, lineno)
        elif type(t) == CtypesFunction:
            # this is the line we override
            self.handle_ctypes_function(
                name, t.restype, t.argtypes, filename, lineno, t.argnames)
        elif declaration.storage != 'static':
            self.handle_ctypes_variable(name, t, filename, lineno)

    def print_preamble(self):
        import time
        print >> self.file, textwrap.dedent("""
            # This content is generated by %(script)s.
            # Wrapper for %(header)s
            from OpenGL import platform, constant
            from ctypes import *
            c_void = None
        """ % {
            'header': self.header,
            'date': time.ctime(),
            'script': __file__,
        }).lstrip()

    def libFromRequires( self, requires  ):
        return 'platform.GL'

    def handle_ctypes_function(self, name, restype, argtypes, filename, lineno, argnames):
        if self.does_emit(name, filename):
            self.emit_type(restype)
            for a in argtypes:
                self.emit_type(a)

            self.all_names.append(name)
            #print >> self.file, '# %s:%d' % (filename, lineno)
            print >> self.file, '''%(name)s = platform.createBaseFunction(
    %(name)r, dll=%(libname)s, resultType=%(returnType)s, 
    argTypes=[%(argTypes)s],
    doc=%(documentation)r, 
    argNames=%(argNames)r,
)'''%dict(
    name = name,
    libname = self.libFromRequires( self.requires  ),
    returnType = restype,
    argTypes = ', '.join([str(a) for a in argtypes]),
    documentation = self.documentation(
        name, argnames, argtypes, restype
    ),
    argNames = [str(x) for x in argnames],
)
            print >> self.file
    def documentation( self, name, argnames, args, returntype ):
        """Customisation point for documenting a given function"""
        return str("%s( %s ) -> %s"%(
            name,
            ", ".join(
                [ '%s(%s)'%( typ, name) for (typ,name) in zip(args,argnames) ]
            ),
            returntype,
        ))

    def handle_ifndef(self, name, filename, lineno):
        if (
            self.requires_prefix and 
            name[:len(self.requires_prefix)] == self.requires_prefix
        ):
            self.requires = name[len(self.requires_prefix):]
            print >> self.file, '# %s (%s:%d)'  % \
                (self.requires, filename, lineno)
    def handle_ctypes_constant(self, name, value, filename, lineno):
        if self.does_emit( name, filename ):
            print >> self.file, '%(name)s = constant.Constant( %(name)r, %(value)r )'%locals()
            self.all_names.append(name)
    def does_emit(self, symbol, filename):
        base = super( GLWrapper, self ).does_emit( symbol, filename )
        if base:
            if self.match_re:
                if self.match_re.match( symbol ):
                    return True 
                else:
                    return False 
        return base

def progress(msg):
    print >> sys.stderr, msg

marker_begin = '# BEGIN GENERATED CONTENT (do not edit below this line)\n'
marker_end = '# END GENERATED CONTENT (do not edit above this line)\n'

class ModuleWrapper(object):
    def __init__(
        self, header, filename,
        prologue='', requires_prefix=None, system_header=None,
        match_re=None,
    ):
        self.header = header
        self.filename = filename
        self.prologue = prologue
        self.requires_prefix = requires_prefix
        self.system_header = system_header
        self.match_re = match_re

    def wrap(self, dir):
        progress('Updating %s...' % self.filename)
        source = read_url(self.header) 
        filename = os.path.join(dir, self.filename)

        prologue = []
        epilogue = []
        state = 'prologue'
        try:
            for line in open(filename):
                if state == 'prologue':
                    prologue.append(line)
                    if line == marker_begin:
                        state = 'generated'
                elif state == 'generated':
                    if line == marker_end:
                        state = 'epilogue'
                        epilogue.append(line)
                elif state == 'epilogue':
                    epilogue.append(line)
        except IOError:
            prologue = [marker_begin]
            epilogue = [marker_end]
            state = 'epilogue'
        if state != 'epilogue':
            raise Exception('File exists, but generated markers are corrupt '
                            'or missing')

        outfile = open(filename, 'w')
        print >> outfile, ''.join(prologue)
        wrapper = GLWrapper(self.header, self.match_re)
        if self.system_header:
            wrapper.preprocessor_parser.system_headers[self.system_header] = \
                source
        header_name = self.system_header or self.header
        wrapper.requires_prefix = self.requires_prefix
        wrapper.begin_output(outfile, 
                             library=None,
                             emit_filenames=(header_name,))
        source = self.prologue + source
        wrapper.wrap(header_name, source)
        wrapper.end_output()
        print >> outfile, ''.join(epilogue)

modules = {
#	'gl':  
#		ModuleWrapper(GL_H, 'gl.py'),
#	'glu': 
#		ModuleWrapper(GLU_H, 'glu.py'),
#	'glext_arb': 
#		ModuleWrapper(GLEXT_ABI_H, 'glext_arb.py', 
#			requires_prefix='GL_', system_header='GL/glext.h',
#			prologue='#define GL_GLEXT_PROTOTYPES\n#include <GL/gl.h>\n'),
#	'glext_nv': 
#		ModuleWrapper(GLEXT_NV_H, 'glext_nv.py',
#			requires_prefix='GL_', system_header='GL/glext.h',
#			prologue='#define GL_GLEXT_PROTOTYPES\n#include <GL/gl.h>\n'),
    'glx': 
        ModuleWrapper(GLX_H, '_GLX.py', 
            requires_prefix='GLX_',
            match_re = re.compile( 'glX|GLX' ),
        ),
    'glx_arb': 
        ModuleWrapper(GLXEXT_ABI_H, '_GLX_ARB.py', requires_prefix='GLX_',
            system_header='GL/glxext.h',
            prologue='#define GLX_GLXEXT_PROTOTYPES\n#include <GL/glx.h>\n',
            match_re = re.compile( 'glX|GLX_' ),
        ),
    'glx_nv': 
        ModuleWrapper(GLXEXT_NV_H, '_GLX_NV.py', requires_prefix='GLX_',
            system_header='GL/glxext.h',
            prologue='#define GLX_GLXEXT_PROTOTYPES\n#include <GL/glx.h>\n',
            match_re = re.compile( 'glX|GLX_' ),
        ),
    'agl':
        ModuleWrapper(AGL_H, 'AGL.py'),
    'wgl':
        ModuleWrapper(WGL_H, '_WGL.py'),
    'wgl_arb':
        ModuleWrapper(WGLEXT_ABI_H, '_WGL_ARB.py', requires_prefix='WGL_',
            prologue='#define WGL_WGLEXT_PROTOTYPES\n'\
                     '#include "%s"\n' % WGL_H.encode('string_escape')),
    'wgl_nv':
        ModuleWrapper(WGLEXT_NV_H, '_WGL_NV.py', requires_prefix='WGL_',
            prologue='#define WGL_WGLEXT_PROTOTYPES\n'\
                     '#include "%s"\n' % WGL_H.encode('string_escape')),
}


if __name__ == '__main__':
    op = optparse.OptionParser()
    op.add_option('-D', '--dir', dest='dir',
                  help='output directory')
    op.add_option('-r', '--refresh-cache', dest='refresh_cache',
                  help='clear cache first', action='store_true')
    options, args = op.parse_args()

    if not options.refresh_cache:
        load_cache()
    else:
        save_cache()

    if not args:
        print >> sys.stderr, 'Specify module(s) to generate:'
        print >> sys.stderr, '  %s' % ' '.join(modules.keys())

    if not options.dir:
        options.dir = os.path.join(script_dir, os.path.pardir, 'OpenGL', 'raw')
    if not os.path.exists(options.dir):
        os.makedirs(options.dir)

    for arg in args:
        if arg not in modules:
            print >> sys.stderr, "Don't know how to make '%s'" % arg
            continue

        modules[arg].wrap(options.dir)



'''OpenGL extension ATI.vertex_array_object

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_vertex_array_object'
_DEPRECATED = False
GL_STATIC_ATI = constant.Constant( 'GL_STATIC_ATI', 0x8760 )
GL_DYNAMIC_ATI = constant.Constant( 'GL_DYNAMIC_ATI', 0x8761 )
GL_PRESERVE_ATI = constant.Constant( 'GL_PRESERVE_ATI', 0x8762 )
GL_DISCARD_ATI = constant.Constant( 'GL_DISCARD_ATI', 0x8763 )
GL_OBJECT_BUFFER_SIZE_ATI = constant.Constant( 'GL_OBJECT_BUFFER_SIZE_ATI', 0x8764 )
GL_OBJECT_BUFFER_USAGE_ATI = constant.Constant( 'GL_OBJECT_BUFFER_USAGE_ATI', 0x8765 )
GL_ARRAY_OBJECT_BUFFER_ATI = constant.Constant( 'GL_ARRAY_OBJECT_BUFFER_ATI', 0x8766 )
GL_ARRAY_OBJECT_OFFSET_ATI = constant.Constant( 'GL_ARRAY_OBJECT_OFFSET_ATI', 0x8767 )
glNewObjectBufferATI = platform.createExtensionFunction( 
'glNewObjectBufferATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLuint, 
argTypes=(constants.GLsizei,ctypes.c_void_p,constants.GLenum,),
doc='glNewObjectBufferATI(GLsizei(size), c_void_p(pointer), GLenum(usage)) -> constants.GLuint',
argNames=('size','pointer','usage',),
deprecated=_DEPRECATED,
)

glIsObjectBufferATI = platform.createExtensionFunction( 
'glIsObjectBufferATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLboolean, 
argTypes=(constants.GLuint,),
doc='glIsObjectBufferATI(GLuint(buffer)) -> constants.GLboolean',
argNames=('buffer',),
deprecated=_DEPRECATED,
)

glUpdateObjectBufferATI = platform.createExtensionFunction( 
'glUpdateObjectBufferATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLsizei,ctypes.c_void_p,constants.GLenum,),
doc='glUpdateObjectBufferATI(GLuint(buffer), GLuint(offset), GLsizei(size), c_void_p(pointer), GLenum(preserve)) -> None',
argNames=('buffer','offset','size','pointer','preserve',),
deprecated=_DEPRECATED,
)

glGetObjectBufferfvATI = platform.createExtensionFunction( 
'glGetObjectBufferfvATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLfloatArray,),
doc='glGetObjectBufferfvATI(GLuint(buffer), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('buffer','pname','params',),
deprecated=_DEPRECATED,
)

glGetObjectBufferivATI = platform.createExtensionFunction( 
'glGetObjectBufferivATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLintArray,),
doc='glGetObjectBufferivATI(GLuint(buffer), GLenum(pname), GLintArray(params)) -> None',
argNames=('buffer','pname','params',),
deprecated=_DEPRECATED,
)

glFreeObjectBufferATI = platform.createExtensionFunction( 
'glFreeObjectBufferATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glFreeObjectBufferATI(GLuint(buffer)) -> None',
argNames=('buffer',),
deprecated=_DEPRECATED,
)

glArrayObjectATI = platform.createExtensionFunction( 
'glArrayObjectATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,constants.GLenum,constants.GLsizei,constants.GLuint,constants.GLuint,),
doc='glArrayObjectATI(GLenum(array), GLint(size), GLenum(type), GLsizei(stride), GLuint(buffer), GLuint(offset)) -> None',
argNames=('array','size','type','stride','buffer','offset',),
deprecated=_DEPRECATED,
)

glGetArrayObjectfvATI = platform.createExtensionFunction( 
'glGetArrayObjectfvATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glGetArrayObjectfvATI(GLenum(array), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('array','pname','params',),
deprecated=_DEPRECATED,
)

glGetArrayObjectivATI = platform.createExtensionFunction( 
'glGetArrayObjectivATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glGetArrayObjectivATI(GLenum(array), GLenum(pname), GLintArray(params)) -> None',
argNames=('array','pname','params',),
deprecated=_DEPRECATED,
)

glVariantArrayObjectATI = platform.createExtensionFunction( 
'glVariantArrayObjectATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,constants.GLsizei,constants.GLuint,constants.GLuint,),
doc='glVariantArrayObjectATI(GLuint(id), GLenum(type), GLsizei(stride), GLuint(buffer), GLuint(offset)) -> None',
argNames=('id','type','stride','buffer','offset',),
deprecated=_DEPRECATED,
)

glGetVariantArrayObjectfvATI = platform.createExtensionFunction( 
'glGetVariantArrayObjectfvATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLfloatArray,),
doc='glGetVariantArrayObjectfvATI(GLuint(id), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('id','pname','params',),
deprecated=_DEPRECATED,
)

glGetVariantArrayObjectivATI = platform.createExtensionFunction( 
'glGetVariantArrayObjectivATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLintArray,),
doc='glGetVariantArrayObjectivATI(GLuint(id), GLenum(pname), GLintArray(params)) -> None',
argNames=('id','pname','params',),
deprecated=_DEPRECATED,
)


def glInitVertexArrayObjectATI():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.convolution_accuracy

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.convolution_accuracy to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds an accuracy hint for convolution.  It
	allows the program to trade off precision for speed.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/convolution_accuracy.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.SGIX.convolution_accuracy import *
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.compiled_vertex_array

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_compiled_vertex_array'
_DEPRECATED = False
GL_ARRAY_ELEMENT_LOCK_FIRST_EXT = constant.Constant( 'GL_ARRAY_ELEMENT_LOCK_FIRST_EXT', 0x81A8 )
glget.addGLGetConstant( GL_ARRAY_ELEMENT_LOCK_FIRST_EXT, (1,) )
GL_ARRAY_ELEMENT_LOCK_COUNT_EXT = constant.Constant( 'GL_ARRAY_ELEMENT_LOCK_COUNT_EXT', 0x81A9 )
glget.addGLGetConstant( GL_ARRAY_ELEMENT_LOCK_COUNT_EXT, (1,) )
glLockArraysEXT = platform.createExtensionFunction( 
'glLockArraysEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLsizei,),
doc='glLockArraysEXT(GLint(first), GLsizei(count)) -> None',
argNames=('first','count',),
deprecated=_DEPRECATED,
)

glUnlockArraysEXT = platform.createExtensionFunction( 
'glUnlockArraysEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(),
doc='glUnlockArraysEXT() -> None',
argNames=(),
deprecated=_DEPRECATED,
)


def glInitCompiledVertexArrayEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension APPLE.float_pixels

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_APPLE_float_pixels'
_DEPRECATED = False
GL_HALF_APPLE = constant.Constant( 'GL_HALF_APPLE', 0x140B )
GL_RGBA_FLOAT32_APPLE = constant.Constant( 'GL_RGBA_FLOAT32_APPLE', 0x8814 )
GL_RGB_FLOAT32_APPLE = constant.Constant( 'GL_RGB_FLOAT32_APPLE', 0x8815 )
GL_ALPHA_FLOAT32_APPLE = constant.Constant( 'GL_ALPHA_FLOAT32_APPLE', 0x8816 )
GL_INTENSITY_FLOAT32_APPLE = constant.Constant( 'GL_INTENSITY_FLOAT32_APPLE', 0x8817 )
GL_LUMINANCE_FLOAT32_APPLE = constant.Constant( 'GL_LUMINANCE_FLOAT32_APPLE', 0x8818 )
GL_LUMINANCE_ALPHA_FLOAT32_APPLE = constant.Constant( 'GL_LUMINANCE_ALPHA_FLOAT32_APPLE', 0x8819 )
GL_RGBA_FLOAT16_APPLE = constant.Constant( 'GL_RGBA_FLOAT16_APPLE', 0x881A )
GL_RGB_FLOAT16_APPLE = constant.Constant( 'GL_RGB_FLOAT16_APPLE', 0x881B )
GL_ALPHA_FLOAT16_APPLE = constant.Constant( 'GL_ALPHA_FLOAT16_APPLE', 0x881C )
GL_INTENSITY_FLOAT16_APPLE = constant.Constant( 'GL_INTENSITY_FLOAT16_APPLE', 0x881D )
GL_LUMINANCE_FLOAT16_APPLE = constant.Constant( 'GL_LUMINANCE_FLOAT16_APPLE', 0x881E )
GL_LUMINANCE_ALPHA_FLOAT16_APPLE = constant.Constant( 'GL_LUMINANCE_ALPHA_FLOAT16_APPLE', 0x881F )
GL_COLOR_FLOAT_APPLE = constant.Constant( 'GL_COLOR_FLOAT_APPLE', 0x8A0F )


def glInitFloatPixelsAPPLE():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.subsample

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_subsample'
_DEPRECATED = False
GL_PACK_SUBSAMPLE_RATE_SGIX = constant.Constant( 'GL_PACK_SUBSAMPLE_RATE_SGIX', 0x85A0 )
GL_UNPACK_SUBSAMPLE_RATE_SGIX = constant.Constant( 'GL_UNPACK_SUBSAMPLE_RATE_SGIX', 0x85A1 )
GL_PIXEL_SUBSAMPLE_4444_SGIX = constant.Constant( 'GL_PIXEL_SUBSAMPLE_4444_SGIX', 0x85A2 )
GL_PIXEL_SUBSAMPLE_2424_SGIX = constant.Constant( 'GL_PIXEL_SUBSAMPLE_2424_SGIX', 0x85A3 )
GL_PIXEL_SUBSAMPLE_4242_SGIX = constant.Constant( 'GL_PIXEL_SUBSAMPLE_4242_SGIX', 0x85A4 )


def glInitSubsampleSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.vertex_buffer_object

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_vertex_buffer_object'
_DEPRECATED = False
GL_BUFFER_SIZE_ARB = constant.Constant( 'GL_BUFFER_SIZE_ARB', 0x8764 )
GL_BUFFER_USAGE_ARB = constant.Constant( 'GL_BUFFER_USAGE_ARB', 0x8765 )
GL_ARRAY_BUFFER_ARB = constant.Constant( 'GL_ARRAY_BUFFER_ARB', 0x8892 )
GL_ELEMENT_ARRAY_BUFFER_ARB = constant.Constant( 'GL_ELEMENT_ARRAY_BUFFER_ARB', 0x8893 )
GL_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_ARRAY_BUFFER_BINDING_ARB', 0x8894 )
glget.addGLGetConstant( GL_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB', 0x8895 )
glget.addGLGetConstant( GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_VERTEX_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_VERTEX_ARRAY_BUFFER_BINDING_ARB', 0x8896 )
glget.addGLGetConstant( GL_VERTEX_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_NORMAL_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_NORMAL_ARRAY_BUFFER_BINDING_ARB', 0x8897 )
glget.addGLGetConstant( GL_NORMAL_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_COLOR_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_COLOR_ARRAY_BUFFER_BINDING_ARB', 0x8898 )
glget.addGLGetConstant( GL_COLOR_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_INDEX_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_INDEX_ARRAY_BUFFER_BINDING_ARB', 0x8899 )
glget.addGLGetConstant( GL_INDEX_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB', 0x889A )
glget.addGLGetConstant( GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB', 0x889B )
glget.addGLGetConstant( GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB', 0x889C )
glget.addGLGetConstant( GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB', 0x889D )
glget.addGLGetConstant( GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB', 0x889E )
glget.addGLGetConstant( GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB, (1,) )
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB', 0x889F )
GL_READ_ONLY_ARB = constant.Constant( 'GL_READ_ONLY_ARB', 0x88B8 )
GL_WRITE_ONLY_ARB = constant.Constant( 'GL_WRITE_ONLY_ARB', 0x88B9 )
GL_READ_WRITE_ARB = constant.Constant( 'GL_READ_WRITE_ARB', 0x88BA )
GL_BUFFER_ACCESS_ARB = constant.Constant( 'GL_BUFFER_ACCESS_ARB', 0x88BB )
GL_BUFFER_MAPPED_ARB = constant.Constant( 'GL_BUFFER_MAPPED_ARB', 0x88BC )
GL_BUFFER_MAP_POINTER_ARB = constant.Constant( 'GL_BUFFER_MAP_POINTER_ARB', 0x88BD )
GL_STREAM_DRAW_ARB = constant.Constant( 'GL_STREAM_DRAW_ARB', 0x88E0 )
GL_STREAM_READ_ARB = constant.Constant( 'GL_STREAM_READ_ARB', 0x88E1 )
GL_STREAM_COPY_ARB = constant.Constant( 'GL_STREAM_COPY_ARB', 0x88E2 )
GL_STATIC_DRAW_ARB = constant.Constant( 'GL_STATIC_DRAW_ARB', 0x88E4 )
GL_STATIC_READ_ARB = constant.Constant( 'GL_STATIC_READ_ARB', 0x88E5 )
GL_STATIC_COPY_ARB = constant.Constant( 'GL_STATIC_COPY_ARB', 0x88E6 )
GL_DYNAMIC_DRAW_ARB = constant.Constant( 'GL_DYNAMIC_DRAW_ARB', 0x88E8 )
GL_DYNAMIC_READ_ARB = constant.Constant( 'GL_DYNAMIC_READ_ARB', 0x88E9 )
GL_DYNAMIC_COPY_ARB = constant.Constant( 'GL_DYNAMIC_COPY_ARB', 0x88EA )
glBindBufferARB = platform.createExtensionFunction( 
'glBindBufferARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,),
doc='glBindBufferARB(GLenum(target), GLuint(buffer)) -> None',
argNames=('target','buffer',),
deprecated=_DEPRECATED,
)

glDeleteBuffersARB = platform.createExtensionFunction( 
'glDeleteBuffersARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLsizei,arrays.GLuintArray,),
doc='glDeleteBuffersARB(GLsizei(n), GLuintArray(buffers)) -> None',
argNames=('n','buffers',),
deprecated=_DEPRECATED,
)

glGenBuffersARB = platform.createExtensionFunction( 
'glGenBuffersARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLsizei,arrays.GLuintArray,),
doc='glGenBuffersARB(GLsizei(n), GLuintArray(buffers)) -> None',
argNames=('n','buffers',),
deprecated=_DEPRECATED,
)

glIsBufferARB = platform.createExtensionFunction( 
'glIsBufferARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLboolean, 
argTypes=(constants.GLuint,),
doc='glIsBufferARB(GLuint(buffer)) -> constants.GLboolean',
argNames=('buffer',),
deprecated=_DEPRECATED,
)

glBufferDataARB = platform.createExtensionFunction( 
'glBufferDataARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizeiptrARB,ctypes.c_void_p,constants.GLenum,),
doc='glBufferDataARB(GLenum(target), GLsizeiptrARB(size), c_void_p(data), GLenum(usage)) -> None',
argNames=('target','size','data','usage',),
deprecated=_DEPRECATED,
)

glBufferSubDataARB = platform.createExtensionFunction( 
'glBufferSubDataARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLintptrARB,constants.GLsizeiptrARB,ctypes.c_void_p,),
doc='glBufferSubDataARB(GLenum(target), GLintptrARB(offset), GLsizeiptrARB(size), c_void_p(data)) -> None',
argNames=('target','offset','size','data',),
deprecated=_DEPRECATED,
)

glGetBufferSubDataARB = platform.createExtensionFunction( 
'glGetBufferSubDataARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLintptrARB,constants.GLsizeiptrARB,ctypes.c_void_p,),
doc='glGetBufferSubDataARB(GLenum(target), GLintptrARB(offset), GLsizeiptrARB(size), c_void_p(data)) -> None',
argNames=('target','offset','size','data',),
deprecated=_DEPRECATED,
)

glMapBufferARB = platform.createExtensionFunction( 
'glMapBufferARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=ctypes.c_void_p, 
argTypes=(constants.GLenum,constants.GLenum,),
doc='glMapBufferARB(GLenum(target), GLenum(access)) -> ctypes.c_void_p',
argNames=('target','access',),
deprecated=_DEPRECATED,
)

glUnmapBufferARB = platform.createExtensionFunction( 
'glUnmapBufferARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLboolean, 
argTypes=(constants.GLenum,),
doc='glUnmapBufferARB(GLenum(target)) -> constants.GLboolean',
argNames=('target',),
deprecated=_DEPRECATED,
)

glGetBufferParameterivARB = platform.createExtensionFunction( 
'glGetBufferParameterivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glGetBufferParameterivARB(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetBufferPointervARB = platform.createExtensionFunction( 
'glGetBufferPointervARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,ctypes.POINTER(ctypes.c_void_p),),
doc='glGetBufferPointervARB(GLenum(target), GLenum(pname), POINTER(ctypes.c_void_p)(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)


def glInitVertexBufferObjectARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_lod_bias

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_lod_bias'
_DEPRECATED = False
GL_MAX_TEXTURE_LOD_BIAS_EXT = constant.Constant( 'GL_MAX_TEXTURE_LOD_BIAS_EXT', 0x84FD )
glget.addGLGetConstant( GL_MAX_TEXTURE_LOD_BIAS_EXT, (1,) )
GL_TEXTURE_FILTER_CONTROL_EXT = constant.Constant( 'GL_TEXTURE_FILTER_CONTROL_EXT', 0x8500 )
GL_TEXTURE_LOD_BIAS_EXT = constant.Constant( 'GL_TEXTURE_LOD_BIAS_EXT', 0x8501 )


def glInitTextureLodBiasEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.transform_feedback

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_transform_feedback'
_DEPRECATED = False
GL_BACK_PRIMARY_COLOR_NV = constant.Constant( 'GL_BACK_PRIMARY_COLOR_NV', 0x8C77 )
GL_BACK_SECONDARY_COLOR_NV = constant.Constant( 'GL_BACK_SECONDARY_COLOR_NV', 0x8C78 )
GL_TEXTURE_COORD_NV = constant.Constant( 'GL_TEXTURE_COORD_NV', 0x8C79 )
GL_CLIP_DISTANCE_NV = constant.Constant( 'GL_CLIP_DISTANCE_NV', 0x8C7A )
GL_VERTEX_ID_NV = constant.Constant( 'GL_VERTEX_ID_NV', 0x8C7B )
GL_PRIMITIVE_ID_NV = constant.Constant( 'GL_PRIMITIVE_ID_NV', 0x8C7C )
GL_GENERIC_ATTRIB_NV = constant.Constant( 'GL_GENERIC_ATTRIB_NV', 0x8C7D )
GL_TRANSFORM_FEEDBACK_ATTRIBS_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_ATTRIBS_NV', 0x8C7E )
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV', 0x8C7F )
glget.addGLGetConstant( GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV, (1,) )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV', 0x8C80 )
GL_ACTIVE_VARYINGS_NV = constant.Constant( 'GL_ACTIVE_VARYINGS_NV', 0x8C81 )
GL_ACTIVE_VARYING_MAX_LENGTH_NV = constant.Constant( 'GL_ACTIVE_VARYING_MAX_LENGTH_NV', 0x8C82 )
GL_TRANSFORM_FEEDBACK_VARYINGS_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_VARYINGS_NV', 0x8C83 )
GL_TRANSFORM_FEEDBACK_BUFFER_START_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_START_NV', 0x8C84 )
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV', 0x8C85 )
GL_TRANSFORM_FEEDBACK_RECORD_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_RECORD_NV', 0x8C86 )
GL_PRIMITIVES_GENERATED_NV = constant.Constant( 'GL_PRIMITIVES_GENERATED_NV', 0x8C87 )
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV', 0x8C88 )
GL_RASTERIZER_DISCARD_NV = constant.Constant( 'GL_RASTERIZER_DISCARD_NV', 0x8C89 )
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV', 0x8C8A )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV', 0x8C8B )
GL_INTERLEAVED_ATTRIBS_NV = constant.Constant( 'GL_INTERLEAVED_ATTRIBS_NV', 0x8C8C )
GL_SEPARATE_ATTRIBS_NV = constant.Constant( 'GL_SEPARATE_ATTRIBS_NV', 0x8C8D )
GL_TRANSFORM_FEEDBACK_BUFFER_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_NV', 0x8C8E )
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV', 0x8C8F )
glBeginTransformFeedbackNV = platform.createExtensionFunction( 
'glBeginTransformFeedbackNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,),
doc='glBeginTransformFeedbackNV(GLenum(primitiveMode)) -> None',
argNames=('primitiveMode',),
deprecated=_DEPRECATED,
)

glEndTransformFeedbackNV = platform.createExtensionFunction( 
'glEndTransformFeedbackNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(),
doc='glEndTransformFeedbackNV() -> None',
argNames=(),
deprecated=_DEPRECATED,
)

glTransformFeedbackAttribsNV = platform.createExtensionFunction( 
'glTransformFeedbackAttribsNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLintArray,constants.GLenum,),
doc='glTransformFeedbackAttribsNV(GLuint(count), GLintArray(attribs), GLenum(bufferMode)) -> None',
argNames=('count','attribs','bufferMode',),
deprecated=_DEPRECATED,
)

glBindBufferRangeNV = platform.createExtensionFunction( 
'glBindBufferRangeNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLuint,constants.GLintptr,constants.GLsizeiptr,),
doc='glBindBufferRangeNV(GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset), GLsizeiptr(size)) -> None',
argNames=('target','index','buffer','offset','size',),
deprecated=_DEPRECATED,
)

glBindBufferOffsetNV = platform.createExtensionFunction( 
'glBindBufferOffsetNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLuint,constants.GLintptr,),
doc='glBindBufferOffsetNV(GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset)) -> None',
argNames=('target','index','buffer','offset',),
deprecated=_DEPRECATED,
)

glBindBufferBaseNV = platform.createExtensionFunction( 
'glBindBufferBaseNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLuint,),
doc='glBindBufferBaseNV(GLenum(target), GLuint(index), GLuint(buffer)) -> None',
argNames=('target','index','buffer',),
deprecated=_DEPRECATED,
)

glTransformFeedbackVaryingsNV = platform.createExtensionFunction( 
'glTransformFeedbackVaryingsNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,ctypes.POINTER( ctypes.POINTER( constants.GLchar )),constants.GLenum,),
doc='glTransformFeedbackVaryingsNV(GLuint(program), GLsizei(count), POINTER( ctypes.POINTER( constants.GLchar ))(varyings), GLenum(bufferMode)) -> None',
argNames=('program','count','varyings','bufferMode',),
deprecated=_DEPRECATED,
)

glActiveVaryingNV = platform.createExtensionFunction( 
'glActiveVaryingNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLcharArray,),
doc='glActiveVaryingNV(GLuint(program), GLcharArray(name)) -> None',
argNames=('program','name',),
deprecated=_DEPRECATED,
)

glGetVaryingLocationNV = platform.createExtensionFunction( 
'glGetVaryingLocationNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLint, 
argTypes=(constants.GLuint,arrays.GLcharArray,),
doc='glGetVaryingLocationNV(GLuint(program), GLcharArray(name)) -> constants.GLint',
argNames=('program','name',),
deprecated=_DEPRECATED,
)

glGetActiveVaryingNV = platform.createExtensionFunction( 
'glGetActiveVaryingNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLsizei,arrays.GLsizeiArray,arrays.GLsizeiArray,arrays.GLuintArray,arrays.GLcharArray,),
doc='glGetActiveVaryingNV(GLuint(program), GLuint(index), GLsizei(bufSize), GLsizeiArray(length), GLsizeiArray(size), GLuintArray(type), GLcharArray(name)) -> None',
argNames=('program','index','bufSize','length','size','type','name',),
deprecated=_DEPRECATED,
)

glGetTransformFeedbackVaryingNV = platform.createExtensionFunction( 
'glGetTransformFeedbackVaryingNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,arrays.GLintArray,),
doc='glGetTransformFeedbackVaryingNV(GLuint(program), GLuint(index), GLintArray(location)) -> None',
argNames=('program','index','location',),
deprecated=_DEPRECATED,
)


def glInitTransformFeedbackNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.index_func

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.index_func to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a way to discard fragments when a comparison
	between the fragment's index value and a reference index fails.  This
	may be used similarly to the alpha test which is available in RGBA mode.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/index_func.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.index_func import *
### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.FfdMask

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_FfdMask'
_DEPRECATED = False
GL_TEXTURE_DEFORMATION_BIT_SGIX = constant.Constant( 'GL_TEXTURE_DEFORMATION_BIT_SGIX', 0x1 )
GL_GEOMETRY_DEFORMATION_BIT_SGIX = constant.Constant( 'GL_GEOMETRY_DEFORMATION_BIT_SGIX', 0x2 )


def glInitFfdmaskSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""ctypes data-pointers as a data-format mechanism
"""
REGISTRY_NAME = 'ctypespointers'
import ctypes, _ctypes

from OpenGL import constants, constant
from OpenGL.arrays import formathandler
import operator

class CtypesPointerHandler( formathandler.FormatHandler ):
    """Ctypes Pointer-type-specific data-type handler for OpenGL
    
    Because pointers do not have size information we can't use
    them for output of data, but they can be used for certain
    types of input...
    """
    @classmethod
    def from_param( cls, value, typeCode=None  ):
        return value
    dataPointer = staticmethod( ctypes.addressof )
    HANDLED_TYPES = (ctypes._Pointer, )
    def voidDataPointer( cls, value ):
        """Given value in a known data-pointer type, return void_p for pointer"""
        return ctypes.cast( value, ctypes.c_void_p )
    def zeros( self, dims, typeCode ):
        """Return Numpy array of zeros in given size"""
        raise NotImplementedError( """Sized output doesn't yet work...""" )
    def ones( self, dims, typeCode='d' ):
        """Return numpy array of ones in given size"""
        raise NotImplementedError( """Haven't got a good ones implementation yet""" )
    def arrayToGLType( self, value ):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        result = ARRAY_TO_GL_TYPE_MAPPING.get( value._type_ )
        if result is not None:
            return result
        raise TypeError(
            """Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
                value._type_, ARRAY_TO_GL_TYPE_MAPPING.keys(), value,
            )
        )
    def arraySize( self, value, typeCode = None ):
        """Given a data-value, calculate dimensions for the array"""
        raise NotImplementedError( """Haven't got an arraySize implementation""" )
    def asArray( self, value, typeCode=None ):
        """Convert given value to an array value of given typeCode"""
        return value
    def unitSize( self, value, typeCode=None ):
        """Determine unit size of an array (if possible)"""
        return 1
    def dimensions( self, value, typeCode=None ):
        """Determine dimensions of the passed array value (if possible)"""
        raise NotImplementedError( """Haven't got a dimensions implementation""" )


ARRAY_TO_GL_TYPE_MAPPING = {
    constants.GLdouble: constants.GL_DOUBLE,
    constants.GLfloat: constants.GL_FLOAT,
    constants.GLint: constants.GL_INT,
    constants.GLuint: constants.GL_UNSIGNED_INT,
    constants.GLshort: constants.GL_SHORT,
    constants.GLushort: constants.GL_UNSIGNED_SHORT,
        
    constants.GLchar: constants.GL_CHAR,
    constants.GLbyte: constants.GL_BYTE,
    constants.GLubyte: constants.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
    constants.GL_DOUBLE: constants.GLdouble,
    constants.GL_FLOAT: constants.GLfloat,
    constants.GL_INT: constants.GLint,
    constants.GL_UNSIGNED_INT: constants.GLuint,
    constants.GL_SHORT: constants.GLshort,
    constants.GL_UNSIGNED_SHORT: constants.GLushort,
        
    constants.GL_CHAR: constants.GLchar,
    constants.GL_BYTE: constants.GLbyte,
    constants.GL_UNSIGNED_BYTE: constants.GLubyte,
}


'''OpenGL extension HP.texture_lighting

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_HP_texture_lighting'
_DEPRECATED = False
GL_TEXTURE_LIGHTING_MODE_HP = constant.Constant( 'GL_TEXTURE_LIGHTING_MODE_HP', 0x8167 )
GL_TEXTURE_POST_SPECULAR_HP = constant.Constant( 'GL_TEXTURE_POST_SPECULAR_HP', 0x8168 )
GL_TEXTURE_PRE_SPECULAR_HP = constant.Constant( 'GL_TEXTURE_PRE_SPECULAR_HP', 0x8169 )


def glInitTextureLightingHP():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension VERSION.GL_2_0

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_2_0 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_2_0.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.VERSION.GL_2_0 import *
### END AUTOGENERATED SECTION
import OpenGL
from OpenGL.raw.GL.ARB.shader_objects import GL_OBJECT_COMPILE_STATUS_ARB as GL_OBJECT_COMPILE_STATUS
from OpenGL.raw.GL.ARB.shader_objects import GL_OBJECT_LINK_STATUS_ARB as GL_OBJECT_LINK_STATUS
from OpenGL.raw.GL.ARB.shader_objects import GL_OBJECT_ACTIVE_UNIFORMS_ARB as GL_OBJECT_ACTIVE_UNIFORMS
from OpenGL.raw.GL.ARB.shader_objects import GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB as GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH
from OpenGL.GL.ARB.shader_objects import glGetInfoLogARB as glGetInfoLog
from OpenGL.lazywrapper import lazy

from OpenGL import converters, error, contextdata
from OpenGL.arrays.arraydatatype import ArrayDatatype, GLenumArray
GL_INFO_LOG_LENGTH = constant.Constant( 'GL_INFO_LOG_LENGTH', 0x8B84 )

glShaderSource = platform.createExtensionFunction( 
    'glShaderSource', dll=platform.GL,
    resultType=None, 
    argTypes=(constants.GLhandle, constants.GLsizei, ctypes.POINTER(ctypes.c_char_p), arrays.GLintArray,),
    doc = 'glShaderSource( GLhandle(shaderObj),[str(string),...]) -> None',
    argNames = ('shaderObj', 'count', 'string', 'length',),
    extension = EXTENSION_NAME,
)
conv = converters.StringLengths( name='string' )
glShaderSource = wrapper.wrapper(
    glShaderSource
).setPyConverter(
    'count' # number of strings
).setPyConverter( 
    'length' # lengths of strings
).setPyConverter(
    'string', conv.stringArray
).setCResolver(
    'string', conv.stringArrayForC,
).setCConverter(
    'length', conv,
).setCConverter(
    'count', conv.totalCount,
)
del conv

for size in (1,2,3,4):
    for format,arrayType in (
        ('f',arrays.GLfloatArray),
        ('i',arrays.GLintArray),
    ):
        name = 'glUniform%(size)s%(format)sv'%globals()
        globals()[name] = arrays.setInputArraySizeType(
            globals()[name],
            None, # don't want to enforce size...
            arrayType, 
            'value',
        )
        del format, arrayType
    del size,name

@lazy( glGetShaderiv )
def glGetShaderiv( baseOperation, shader, pname, status=None ):
    """Retrieve the integer parameter for the given shader
    
    shader -- shader ID to query 
    pname -- parameter name 
    status -- pointer to integer to receive status or None to 
        return the parameter as an integer value 
    
    returns 
        integer if status parameter is None
        status if status parameter is not None
    """
    if status is None:
        status = arrays.GLintArray.zeros( (1,))
        status[0] = 1 
        baseOperation(
            shader, pname, status
        )
        return status[0]
    else:
        baseOperation(
            shader, pname, status
        )
        return status
@lazy( glGetProgramiv )
def glGetProgramiv( baseOperation, program, pname, params=None ):
    """Will automatically allocate params if not provided"""
    if params is None:
        params = arrays.GLintArray.zeros( (1,))
        baseOperation( program, pname, params )
        return params[0]
    else:
        baseOperation( program,pname, params )
        return params

def _afterCheck( key ):
    """Generate an error-checking function for compilation operations"""
    if key == GL_OBJECT_COMPILE_STATUS:
        getter = glGetShaderiv
    else:
        getter = glGetProgramiv
    def GLSLCheckError( 
        result,
        baseOperation=None,
        cArguments=None,
        *args
    ):
        result = error.glCheckError( result, baseOperation, cArguments, *args )
        status = ctypes.c_int()
        getter( cArguments[0], key, ctypes.byref(status))
        status = status.value
        if not status:
            raise error.GLError( 
                result = result,
                baseOperation = baseOperation,
                cArguments = cArguments,
                description= glGetInfoLog( cArguments[0] )
            )
        return result
    return GLSLCheckError

if OpenGL.ERROR_CHECKING:
    glCompileShader.errcheck = _afterCheck( GL_OBJECT_COMPILE_STATUS )
if OpenGL.ERROR_CHECKING:
    glLinkProgram.errcheck = _afterCheck( GL_OBJECT_LINK_STATUS )
## Not sure why, but these give invalid operation :(
##if glValidateProgram and OpenGL.ERROR_CHECKING:
##	glValidateProgram.errcheck = _afterCheck( GL_OBJECT_VALIDATE_STATUS )

@lazy( glGetShaderInfoLog )
def glGetShaderInfoLog( baseOperation, obj ):
    """Retrieve the shader's error messages as a Python string
    
    returns string which is '' if no message
    """
    length = int(glGetShaderiv(obj, GL_INFO_LOG_LENGTH))
    if length > 0:
        log = ctypes.create_string_buffer(length)
        baseOperation(obj, length, None, log)
        return log.value.strip('\000') # null-termination
    return ''
@lazy( glGetProgramInfoLog )
def glGetProgramInfoLog( baseOperation, obj ):
    """Retrieve the shader program's error messages as a Python string
    
    returns string which is '' if no message
    """
    length = int(glGetProgramiv(obj, GL_INFO_LOG_LENGTH))
    if length > 0:
        log = ctypes.create_string_buffer(length)
        baseOperation(obj, length, None, log)
        return log.value.strip('\000') # null-termination
    return ''

@lazy( glGetAttachedShaders )
def glGetAttachedShaders( baseOperation, obj ):
    """Retrieve the attached objects as an array of GLhandle instances"""
    length= glGetProgramiv( obj, GL_ATTACHED_SHADERS )
    if length > 0:
        storage = arrays.GLuintArray.zeros( (length,))
        baseOperation( obj, length, None, storage )
        return storage
    return arrays.GLuintArray.zeros( (0,))


@lazy( glGetShaderSource )
def glGetShaderSource( baseOperation, obj ):
    """Retrieve the program/shader's source code as a Python string
    
    returns string which is '' if no source code
    """
    length = int(glGetShaderiv(obj, GL_OBJECT_SHADER_SOURCE_LENGTH))
    if length > 0:
        source = ctypes.create_string_buffer(length)
        baseOperation(obj, length, None, source)
        return source.value.strip('\000') # null-termination
    return ''

@lazy( glGetActiveUniform )
def glGetActiveUniform(baseOperation,program, index):
    """Retrieve the name, size and type of the uniform of the index in the program"""
    max_index = int(glGetProgramiv( program, GL_OBJECT_ACTIVE_UNIFORMS ))
    length = int(glGetProgramiv( program, GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH))
    if index < max_index and index >= 0:
        if length > 0:
            name = ctypes.create_string_buffer(length)
            size = arrays.GLintArray.zeros( (1,))
            gl_type = arrays.GLenumArray.zeros( (1,))
            namelen = arrays.GLsizeiArray.zeros( (1,))
            baseOperation(program, index, length, namelen, size, gl_type, name)
            return name.value[:int(namelen[0])], size[0], gl_type[0]
        raise ValueError( """No currently specified uniform names""" )
    raise IndexError, 'Index %s out of range 0 to %i' % (index, max_index - 1, )

@lazy( glGetUniformLocation )
def glGetUniformLocation( baseOperation, program, name ):
    """Check that name is a string with a null byte at the end of it"""
    if not name:
        raise ValueError( """Non-null name required""" )
    elif name[-1] != '\000':
        name = name + '\000'
    return baseOperation( program, name )
@lazy( glGetAttribLocation )
def glGetAttribLocation( baseOperation, program, name ):
    """Check that name is a string with a null byte at the end of it"""
    if not name:
        raise ValueError( """Non-null name required""" )
    elif name[-1] != '\000':
        name = name + '\000'
    return baseOperation( program, name )

@lazy( glVertexAttribPointer )
def glVertexAttribPointer( 
    baseOperation, index, size, type,
    normalized, stride, pointer,
):
    """Set an attribute pointer for a given shader (index)
    
    index -- the index of the generic vertex to bind, see 
        glGetAttribLocation for retrieval of the value,
        note that index is a global variable, not per-shader
    size -- number of basic elements per record, 1,2,3, or 4
    type -- enum constant for data-type 
    normalized -- whether to perform int to float 
        normalization on integer-type values
    stride -- stride in machine units (bytes) between 
        consecutive records, normally used to create 
        "interleaved" arrays 
    pointer -- data-pointer which provides the data-values,
        normally a vertex-buffer-object or offset into the 
        same.
    
    This implementation stores a copy of the data-pointer 
    in the contextdata structure in order to prevent null-
    reference errors in the renderer.
    """
    array = ArrayDatatype.asArray( pointer )
    key = ('vertex-attrib',index)
    contextdata.setValue( key, array )
    return baseOperation(
        index, size, type,
        normalized, stride, 
        ArrayDatatype.voidDataPointer( array ) 
    )

@lazy( glDrawBuffers )
def glDrawBuffers( baseOperation, n=None, bufs=None ):
    """glDrawBuffers( bufs ) -> bufs 
    
    Wrapper will calculate n from dims of bufs if only 
    one argument is provided...
    """
    if bufs is None:
        bufs = n
        n = None
    bufs = arrays.GLenumArray.asArray( bufs )
    if n is None:
        n = arrays.GLenumArray.arraySize( bufs )
    return baseOperation( n,bufs )



'''OpenGL extension ARB.vertex_program

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_vertex_program'
_DEPRECATED = False
GL_COLOR_SUM_ARB = constant.Constant( 'GL_COLOR_SUM_ARB', 0x8458 )
glget.addGLGetConstant( GL_COLOR_SUM_ARB, (1,) )
GL_VERTEX_PROGRAM_ARB = constant.Constant( 'GL_VERTEX_PROGRAM_ARB', 0x8620 )
glget.addGLGetConstant( GL_VERTEX_PROGRAM_ARB, (1,) )
GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB', 0x8622 )
GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB', 0x8623 )
GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB', 0x8624 )
GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB', 0x8625 )
GL_CURRENT_VERTEX_ATTRIB_ARB = constant.Constant( 'GL_CURRENT_VERTEX_ATTRIB_ARB', 0x8626 )
GL_PROGRAM_LENGTH_ARB = constant.Constant( 'GL_PROGRAM_LENGTH_ARB', 0x8627 )
GL_PROGRAM_STRING_ARB = constant.Constant( 'GL_PROGRAM_STRING_ARB', 0x8628 )
GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB = constant.Constant( 'GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB', 0x862E )
glget.addGLGetConstant( GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB, (1,) )
GL_MAX_PROGRAM_MATRICES_ARB = constant.Constant( 'GL_MAX_PROGRAM_MATRICES_ARB', 0x862F )
glget.addGLGetConstant( GL_MAX_PROGRAM_MATRICES_ARB, (1,) )
GL_CURRENT_MATRIX_STACK_DEPTH_ARB = constant.Constant( 'GL_CURRENT_MATRIX_STACK_DEPTH_ARB', 0x8640 )
glget.addGLGetConstant( GL_CURRENT_MATRIX_STACK_DEPTH_ARB, (1,) )
GL_CURRENT_MATRIX_ARB = constant.Constant( 'GL_CURRENT_MATRIX_ARB', 0x8641 )
glget.addGLGetConstant( GL_CURRENT_MATRIX_ARB, (4,4) )
GL_VERTEX_PROGRAM_POINT_SIZE_ARB = constant.Constant( 'GL_VERTEX_PROGRAM_POINT_SIZE_ARB', 0x8642 )
glget.addGLGetConstant( GL_VERTEX_PROGRAM_POINT_SIZE_ARB, (1,) )
GL_VERTEX_PROGRAM_TWO_SIDE_ARB = constant.Constant( 'GL_VERTEX_PROGRAM_TWO_SIDE_ARB', 0x8643 )
glget.addGLGetConstant( GL_VERTEX_PROGRAM_TWO_SIDE_ARB, (1,) )
GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB', 0x8645 )
GL_PROGRAM_ERROR_POSITION_ARB = constant.Constant( 'GL_PROGRAM_ERROR_POSITION_ARB', 0x864B )
glget.addGLGetConstant( GL_PROGRAM_ERROR_POSITION_ARB, (1,) )
GL_PROGRAM_BINDING_ARB = constant.Constant( 'GL_PROGRAM_BINDING_ARB', 0x8677 )
GL_MAX_VERTEX_ATTRIBS_ARB = constant.Constant( 'GL_MAX_VERTEX_ATTRIBS_ARB', 0x8869 )
glget.addGLGetConstant( GL_MAX_VERTEX_ATTRIBS_ARB, (1,) )
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB', 0x886A )
GL_PROGRAM_ERROR_STRING_ARB = constant.Constant( 'GL_PROGRAM_ERROR_STRING_ARB', 0x8874 )
GL_PROGRAM_FORMAT_ASCII_ARB = constant.Constant( 'GL_PROGRAM_FORMAT_ASCII_ARB', 0x8875 )
GL_PROGRAM_FORMAT_ARB = constant.Constant( 'GL_PROGRAM_FORMAT_ARB', 0x8876 )
GL_PROGRAM_INSTRUCTIONS_ARB = constant.Constant( 'GL_PROGRAM_INSTRUCTIONS_ARB', 0x88A0 )
GL_MAX_PROGRAM_INSTRUCTIONS_ARB = constant.Constant( 'GL_MAX_PROGRAM_INSTRUCTIONS_ARB', 0x88A1 )
GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB = constant.Constant( 'GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB', 0x88A2 )
GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB = constant.Constant( 'GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB', 0x88A3 )
GL_PROGRAM_TEMPORARIES_ARB = constant.Constant( 'GL_PROGRAM_TEMPORARIES_ARB', 0x88A4 )
GL_MAX_PROGRAM_TEMPORARIES_ARB = constant.Constant( 'GL_MAX_PROGRAM_TEMPORARIES_ARB', 0x88A5 )
GL_PROGRAM_NATIVE_TEMPORARIES_ARB = constant.Constant( 'GL_PROGRAM_NATIVE_TEMPORARIES_ARB', 0x88A6 )
GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB = constant.Constant( 'GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB', 0x88A7 )
GL_PROGRAM_PARAMETERS_ARB = constant.Constant( 'GL_PROGRAM_PARAMETERS_ARB', 0x88A8 )
GL_MAX_PROGRAM_PARAMETERS_ARB = constant.Constant( 'GL_MAX_PROGRAM_PARAMETERS_ARB', 0x88A9 )
GL_PROGRAM_NATIVE_PARAMETERS_ARB = constant.Constant( 'GL_PROGRAM_NATIVE_PARAMETERS_ARB', 0x88AA )
GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB = constant.Constant( 'GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB', 0x88AB )
GL_PROGRAM_ATTRIBS_ARB = constant.Constant( 'GL_PROGRAM_ATTRIBS_ARB', 0x88AC )
GL_MAX_PROGRAM_ATTRIBS_ARB = constant.Constant( 'GL_MAX_PROGRAM_ATTRIBS_ARB', 0x88AD )
GL_PROGRAM_NATIVE_ATTRIBS_ARB = constant.Constant( 'GL_PROGRAM_NATIVE_ATTRIBS_ARB', 0x88AE )
GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB = constant.Constant( 'GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB', 0x88AF )
GL_PROGRAM_ADDRESS_REGISTERS_ARB = constant.Constant( 'GL_PROGRAM_ADDRESS_REGISTERS_ARB', 0x88B0 )
GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB = constant.Constant( 'GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB', 0x88B1 )
GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = constant.Constant( 'GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB', 0x88B2 )
GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = constant.Constant( 'GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB', 0x88B3 )
GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB = constant.Constant( 'GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB', 0x88B4 )
GL_MAX_PROGRAM_ENV_PARAMETERS_ARB = constant.Constant( 'GL_MAX_PROGRAM_ENV_PARAMETERS_ARB', 0x88B5 )
GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB = constant.Constant( 'GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB', 0x88B6 )
GL_TRANSPOSE_CURRENT_MATRIX_ARB = constant.Constant( 'GL_TRANSPOSE_CURRENT_MATRIX_ARB', 0x88B7 )
glget.addGLGetConstant( GL_TRANSPOSE_CURRENT_MATRIX_ARB, (4,4) )
GL_MATRIX0_ARB = constant.Constant( 'GL_MATRIX0_ARB', 0x88C0 )
GL_MATRIX1_ARB = constant.Constant( 'GL_MATRIX1_ARB', 0x88C1 )
GL_MATRIX2_ARB = constant.Constant( 'GL_MATRIX2_ARB', 0x88C2 )
GL_MATRIX3_ARB = constant.Constant( 'GL_MATRIX3_ARB', 0x88C3 )
GL_MATRIX4_ARB = constant.Constant( 'GL_MATRIX4_ARB', 0x88C4 )
GL_MATRIX5_ARB = constant.Constant( 'GL_MATRIX5_ARB', 0x88C5 )
GL_MATRIX6_ARB = constant.Constant( 'GL_MATRIX6_ARB', 0x88C6 )
GL_MATRIX7_ARB = constant.Constant( 'GL_MATRIX7_ARB', 0x88C7 )
GL_MATRIX8_ARB = constant.Constant( 'GL_MATRIX8_ARB', 0x88C8 )
GL_MATRIX9_ARB = constant.Constant( 'GL_MATRIX9_ARB', 0x88C9 )
GL_MATRIX10_ARB = constant.Constant( 'GL_MATRIX10_ARB', 0x88CA )
GL_MATRIX11_ARB = constant.Constant( 'GL_MATRIX11_ARB', 0x88CB )
GL_MATRIX12_ARB = constant.Constant( 'GL_MATRIX12_ARB', 0x88CC )
GL_MATRIX13_ARB = constant.Constant( 'GL_MATRIX13_ARB', 0x88CD )
GL_MATRIX14_ARB = constant.Constant( 'GL_MATRIX14_ARB', 0x88CE )
GL_MATRIX15_ARB = constant.Constant( 'GL_MATRIX15_ARB', 0x88CF )
GL_MATRIX16_ARB = constant.Constant( 'GL_MATRIX16_ARB', 0x88D0 )
GL_MATRIX17_ARB = constant.Constant( 'GL_MATRIX17_ARB', 0x88D1 )
GL_MATRIX18_ARB = constant.Constant( 'GL_MATRIX18_ARB', 0x88D2 )
GL_MATRIX19_ARB = constant.Constant( 'GL_MATRIX19_ARB', 0x88D3 )
GL_MATRIX20_ARB = constant.Constant( 'GL_MATRIX20_ARB', 0x88D4 )
GL_MATRIX21_ARB = constant.Constant( 'GL_MATRIX21_ARB', 0x88D5 )
GL_MATRIX22_ARB = constant.Constant( 'GL_MATRIX22_ARB', 0x88D6 )
GL_MATRIX23_ARB = constant.Constant( 'GL_MATRIX23_ARB', 0x88D7 )
GL_MATRIX24_ARB = constant.Constant( 'GL_MATRIX24_ARB', 0x88D8 )
GL_MATRIX25_ARB = constant.Constant( 'GL_MATRIX25_ARB', 0x88D9 )
GL_MATRIX26_ARB = constant.Constant( 'GL_MATRIX26_ARB', 0x88DA )
GL_MATRIX27_ARB = constant.Constant( 'GL_MATRIX27_ARB', 0x88DB )
GL_MATRIX28_ARB = constant.Constant( 'GL_MATRIX28_ARB', 0x88DC )
GL_MATRIX29_ARB = constant.Constant( 'GL_MATRIX29_ARB', 0x88DD )
GL_MATRIX30_ARB = constant.Constant( 'GL_MATRIX30_ARB', 0x88DE )
GL_MATRIX31_ARB = constant.Constant( 'GL_MATRIX31_ARB', 0x88DF )
glVertexAttrib1dARB = platform.createExtensionFunction( 
'glVertexAttrib1dARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLdouble,),
doc='glVertexAttrib1dARB(GLuint(index), GLdouble(x)) -> None',
argNames=('index','x',),
deprecated=_DEPRECATED,
)

glVertexAttrib1dvARB = platform.createExtensionFunction( 
'glVertexAttrib1dvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLdoubleArray,),
doc='glVertexAttrib1dvARB(GLuint(index), GLdoubleArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib1fARB = platform.createExtensionFunction( 
'glVertexAttrib1fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLfloat,),
doc='glVertexAttrib1fARB(GLuint(index), GLfloat(x)) -> None',
argNames=('index','x',),
deprecated=_DEPRECATED,
)

glVertexAttrib1fvARB = platform.createExtensionFunction( 
'glVertexAttrib1fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLfloatArray,),
doc='glVertexAttrib1fvARB(GLuint(index), GLfloatArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib1sARB = platform.createExtensionFunction( 
'glVertexAttrib1sARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLshort,),
doc='glVertexAttrib1sARB(GLuint(index), GLshort(x)) -> None',
argNames=('index','x',),
deprecated=_DEPRECATED,
)

glVertexAttrib1svARB = platform.createExtensionFunction( 
'glVertexAttrib1svARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLshortArray,),
doc='glVertexAttrib1svARB(GLuint(index), GLshortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib2dARB = platform.createExtensionFunction( 
'glVertexAttrib2dARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLdouble,constants.GLdouble,),
doc='glVertexAttrib2dARB(GLuint(index), GLdouble(x), GLdouble(y)) -> None',
argNames=('index','x','y',),
deprecated=_DEPRECATED,
)

glVertexAttrib2dvARB = platform.createExtensionFunction( 
'glVertexAttrib2dvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLdoubleArray,),
doc='glVertexAttrib2dvARB(GLuint(index), GLdoubleArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib2fARB = platform.createExtensionFunction( 
'glVertexAttrib2fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLfloat,constants.GLfloat,),
doc='glVertexAttrib2fARB(GLuint(index), GLfloat(x), GLfloat(y)) -> None',
argNames=('index','x','y',),
deprecated=_DEPRECATED,
)

glVertexAttrib2fvARB = platform.createExtensionFunction( 
'glVertexAttrib2fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLfloatArray,),
doc='glVertexAttrib2fvARB(GLuint(index), GLfloatArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib2sARB = platform.createExtensionFunction( 
'glVertexAttrib2sARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLshort,constants.GLshort,),
doc='glVertexAttrib2sARB(GLuint(index), GLshort(x), GLshort(y)) -> None',
argNames=('index','x','y',),
deprecated=_DEPRECATED,
)

glVertexAttrib2svARB = platform.createExtensionFunction( 
'glVertexAttrib2svARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLshortArray,),
doc='glVertexAttrib2svARB(GLuint(index), GLshortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib3dARB = platform.createExtensionFunction( 
'glVertexAttrib3dARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLdouble,constants.GLdouble,constants.GLdouble,),
doc='glVertexAttrib3dARB(GLuint(index), GLdouble(x), GLdouble(y), GLdouble(z)) -> None',
argNames=('index','x','y','z',),
deprecated=_DEPRECATED,
)

glVertexAttrib3dvARB = platform.createExtensionFunction( 
'glVertexAttrib3dvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLdoubleArray,),
doc='glVertexAttrib3dvARB(GLuint(index), GLdoubleArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib3fARB = platform.createExtensionFunction( 
'glVertexAttrib3fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glVertexAttrib3fARB(GLuint(index), GLfloat(x), GLfloat(y), GLfloat(z)) -> None',
argNames=('index','x','y','z',),
deprecated=_DEPRECATED,
)

glVertexAttrib3fvARB = platform.createExtensionFunction( 
'glVertexAttrib3fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLfloatArray,),
doc='glVertexAttrib3fvARB(GLuint(index), GLfloatArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib3sARB = platform.createExtensionFunction( 
'glVertexAttrib3sARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLshort,constants.GLshort,constants.GLshort,),
doc='glVertexAttrib3sARB(GLuint(index), GLshort(x), GLshort(y), GLshort(z)) -> None',
argNames=('index','x','y','z',),
deprecated=_DEPRECATED,
)

glVertexAttrib3svARB = platform.createExtensionFunction( 
'glVertexAttrib3svARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLshortArray,),
doc='glVertexAttrib3svARB(GLuint(index), GLshortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4NbvARB = platform.createExtensionFunction( 
'glVertexAttrib4NbvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLbyteArray,),
doc='glVertexAttrib4NbvARB(GLuint(index), GLbyteArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4NivARB = platform.createExtensionFunction( 
'glVertexAttrib4NivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLintArray,),
doc='glVertexAttrib4NivARB(GLuint(index), GLintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4NsvARB = platform.createExtensionFunction( 
'glVertexAttrib4NsvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLshortArray,),
doc='glVertexAttrib4NsvARB(GLuint(index), GLshortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4NubARB = platform.createExtensionFunction( 
'glVertexAttrib4NubARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLubyte,constants.GLubyte,constants.GLubyte,constants.GLubyte,),
doc='glVertexAttrib4NubARB(GLuint(index), GLubyte(x), GLubyte(y), GLubyte(z), GLubyte(w)) -> None',
argNames=('index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glVertexAttrib4NubvARB = platform.createExtensionFunction( 
'glVertexAttrib4NubvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLubyteArray,),
doc='glVertexAttrib4NubvARB(GLuint(index), GLubyteArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4NuivARB = platform.createExtensionFunction( 
'glVertexAttrib4NuivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLuintArray,),
doc='glVertexAttrib4NuivARB(GLuint(index), GLuintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4NusvARB = platform.createExtensionFunction( 
'glVertexAttrib4NusvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLushortArray,),
doc='glVertexAttrib4NusvARB(GLuint(index), GLushortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4bvARB = platform.createExtensionFunction( 
'glVertexAttrib4bvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLbyteArray,),
doc='glVertexAttrib4bvARB(GLuint(index), GLbyteArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4dARB = platform.createExtensionFunction( 
'glVertexAttrib4dARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLdouble,constants.GLdouble,constants.GLdouble,constants.GLdouble,),
doc='glVertexAttrib4dARB(GLuint(index), GLdouble(x), GLdouble(y), GLdouble(z), GLdouble(w)) -> None',
argNames=('index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glVertexAttrib4dvARB = platform.createExtensionFunction( 
'glVertexAttrib4dvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLdoubleArray,),
doc='glVertexAttrib4dvARB(GLuint(index), GLdoubleArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4fARB = platform.createExtensionFunction( 
'glVertexAttrib4fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLfloat,constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glVertexAttrib4fARB(GLuint(index), GLfloat(x), GLfloat(y), GLfloat(z), GLfloat(w)) -> None',
argNames=('index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glVertexAttrib4fvARB = platform.createExtensionFunction( 
'glVertexAttrib4fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLfloatArray,),
doc='glVertexAttrib4fvARB(GLuint(index), GLfloatArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4ivARB = platform.createExtensionFunction( 
'glVertexAttrib4ivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLintArray,),
doc='glVertexAttrib4ivARB(GLuint(index), GLintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4sARB = platform.createExtensionFunction( 
'glVertexAttrib4sARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLshort,constants.GLshort,constants.GLshort,constants.GLshort,),
doc='glVertexAttrib4sARB(GLuint(index), GLshort(x), GLshort(y), GLshort(z), GLshort(w)) -> None',
argNames=('index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glVertexAttrib4svARB = platform.createExtensionFunction( 
'glVertexAttrib4svARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLshortArray,),
doc='glVertexAttrib4svARB(GLuint(index), GLshortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4ubvARB = platform.createExtensionFunction( 
'glVertexAttrib4ubvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLubyteArray,),
doc='glVertexAttrib4ubvARB(GLuint(index), GLubyteArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4uivARB = platform.createExtensionFunction( 
'glVertexAttrib4uivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLuintArray,),
doc='glVertexAttrib4uivARB(GLuint(index), GLuintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttrib4usvARB = platform.createExtensionFunction( 
'glVertexAttrib4usvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLushortArray,),
doc='glVertexAttrib4usvARB(GLuint(index), GLushortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribPointerARB = platform.createExtensionFunction( 
'glVertexAttribPointerARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLint,constants.GLenum,constants.GLboolean,constants.GLsizei,ctypes.c_void_p,),
doc='glVertexAttribPointerARB(GLuint(index), GLint(size), GLenum(type), GLboolean(normalized), GLsizei(stride), c_void_p(pointer)) -> None',
argNames=('index','size','type','normalized','stride','pointer',),
deprecated=_DEPRECATED,
)

glEnableVertexAttribArrayARB = platform.createExtensionFunction( 
'glEnableVertexAttribArrayARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glEnableVertexAttribArrayARB(GLuint(index)) -> None',
argNames=('index',),
deprecated=_DEPRECATED,
)

glDisableVertexAttribArrayARB = platform.createExtensionFunction( 
'glDisableVertexAttribArrayARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glDisableVertexAttribArrayARB(GLuint(index)) -> None',
argNames=('index',),
deprecated=_DEPRECATED,
)

glProgramStringARB = platform.createExtensionFunction( 
'glProgramStringARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLsizei,ctypes.c_void_p,),
doc='glProgramStringARB(GLenum(target), GLenum(format), GLsizei(len), c_void_p(string)) -> None',
argNames=('target','format','len','string',),
deprecated=_DEPRECATED,
)

glBindProgramARB = platform.createExtensionFunction( 
'glBindProgramARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,),
doc='glBindProgramARB(GLenum(target), GLuint(program)) -> None',
argNames=('target','program',),
deprecated=_DEPRECATED,
)

glDeleteProgramsARB = platform.createExtensionFunction( 
'glDeleteProgramsARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLsizei,arrays.GLuintArray,),
doc='glDeleteProgramsARB(GLsizei(n), GLuintArray(programs)) -> None',
argNames=('n','programs',),
deprecated=_DEPRECATED,
)

glGenProgramsARB = platform.createExtensionFunction( 
'glGenProgramsARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLsizei,arrays.GLuintArray,),
doc='glGenProgramsARB(GLsizei(n), GLuintArray(programs)) -> None',
argNames=('n','programs',),
deprecated=_DEPRECATED,
)

glProgramEnvParameter4dARB = platform.createExtensionFunction( 
'glProgramEnvParameter4dARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLdouble,constants.GLdouble,constants.GLdouble,constants.GLdouble,),
doc='glProgramEnvParameter4dARB(GLenum(target), GLuint(index), GLdouble(x), GLdouble(y), GLdouble(z), GLdouble(w)) -> None',
argNames=('target','index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glProgramEnvParameter4dvARB = platform.createExtensionFunction( 
'glProgramEnvParameter4dvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,arrays.GLdoubleArray,),
doc='glProgramEnvParameter4dvARB(GLenum(target), GLuint(index), GLdoubleArray(params)) -> None',
argNames=('target','index','params',),
deprecated=_DEPRECATED,
)

glProgramEnvParameter4fARB = platform.createExtensionFunction( 
'glProgramEnvParameter4fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLfloat,constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glProgramEnvParameter4fARB(GLenum(target), GLuint(index), GLfloat(x), GLfloat(y), GLfloat(z), GLfloat(w)) -> None',
argNames=('target','index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glProgramEnvParameter4fvARB = platform.createExtensionFunction( 
'glProgramEnvParameter4fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,arrays.GLfloatArray,),
doc='glProgramEnvParameter4fvARB(GLenum(target), GLuint(index), GLfloatArray(params)) -> None',
argNames=('target','index','params',),
deprecated=_DEPRECATED,
)

glProgramLocalParameter4dARB = platform.createExtensionFunction( 
'glProgramLocalParameter4dARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLdouble,constants.GLdouble,constants.GLdouble,constants.GLdouble,),
doc='glProgramLocalParameter4dARB(GLenum(target), GLuint(index), GLdouble(x), GLdouble(y), GLdouble(z), GLdouble(w)) -> None',
argNames=('target','index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glProgramLocalParameter4dvARB = platform.createExtensionFunction( 
'glProgramLocalParameter4dvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,arrays.GLdoubleArray,),
doc='glProgramLocalParameter4dvARB(GLenum(target), GLuint(index), GLdoubleArray(params)) -> None',
argNames=('target','index','params',),
deprecated=_DEPRECATED,
)

glProgramLocalParameter4fARB = platform.createExtensionFunction( 
'glProgramLocalParameter4fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLfloat,constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glProgramLocalParameter4fARB(GLenum(target), GLuint(index), GLfloat(x), GLfloat(y), GLfloat(z), GLfloat(w)) -> None',
argNames=('target','index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glProgramLocalParameter4fvARB = platform.createExtensionFunction( 
'glProgramLocalParameter4fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,arrays.GLfloatArray,),
doc='glProgramLocalParameter4fvARB(GLenum(target), GLuint(index), GLfloatArray(params)) -> None',
argNames=('target','index','params',),
deprecated=_DEPRECATED,
)

glGetProgramEnvParameterdvARB = platform.createExtensionFunction( 
'glGetProgramEnvParameterdvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,arrays.GLdoubleArray,),
doc='glGetProgramEnvParameterdvARB(GLenum(target), GLuint(index), GLdoubleArray(params)) -> None',
argNames=('target','index','params',),
deprecated=_DEPRECATED,
)

glGetProgramEnvParameterfvARB = platform.createExtensionFunction( 
'glGetProgramEnvParameterfvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,arrays.GLfloatArray,),
doc='glGetProgramEnvParameterfvARB(GLenum(target), GLuint(index), GLfloatArray(params)) -> None',
argNames=('target','index','params',),
deprecated=_DEPRECATED,
)

glGetProgramLocalParameterdvARB = platform.createExtensionFunction( 
'glGetProgramLocalParameterdvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,arrays.GLdoubleArray,),
doc='glGetProgramLocalParameterdvARB(GLenum(target), GLuint(index), GLdoubleArray(params)) -> None',
argNames=('target','index','params',),
deprecated=_DEPRECATED,
)

glGetProgramLocalParameterfvARB = platform.createExtensionFunction( 
'glGetProgramLocalParameterfvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,arrays.GLfloatArray,),
doc='glGetProgramLocalParameterfvARB(GLenum(target), GLuint(index), GLfloatArray(params)) -> None',
argNames=('target','index','params',),
deprecated=_DEPRECATED,
)

glGetProgramivARB = platform.createExtensionFunction( 
'glGetProgramivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glGetProgramivARB(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetProgramStringARB = platform.createExtensionFunction( 
'glGetProgramStringARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glGetProgramStringARB(GLenum(target), GLenum(pname), c_void_p(string)) -> None',
argNames=('target','pname','string',),
deprecated=_DEPRECATED,
)

glGetVertexAttribdvARB = platform.createExtensionFunction( 
'glGetVertexAttribdvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLdoubleArray,),
doc='glGetVertexAttribdvARB(GLuint(index), GLenum(pname), GLdoubleArray(params)) -> None',
argNames=('index','pname','params',),
deprecated=_DEPRECATED,
)

glGetVertexAttribfvARB = platform.createExtensionFunction( 
'glGetVertexAttribfvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLfloatArray,),
doc='glGetVertexAttribfvARB(GLuint(index), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('index','pname','params',),
deprecated=_DEPRECATED,
)

glGetVertexAttribivARB = platform.createExtensionFunction( 
'glGetVertexAttribivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLintArray,),
doc='glGetVertexAttribivARB(GLuint(index), GLenum(pname), GLintArray(params)) -> None',
argNames=('index','pname','params',),
deprecated=_DEPRECATED,
)

glGetVertexAttribPointervARB = platform.createExtensionFunction( 
'glGetVertexAttribPointervARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,ctypes.POINTER(ctypes.c_void_p),),
doc='glGetVertexAttribPointervARB(GLuint(index), GLenum(pname), POINTER(ctypes.c_void_p)(pointer)) -> None',
argNames=('index','pname','pointer',),
deprecated=_DEPRECATED,
)

glIsProgramARB = platform.createExtensionFunction( 
'glIsProgramARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLboolean, 
argTypes=(constants.GLuint,),
doc='glIsProgramARB(GLuint(program)) -> constants.GLboolean',
argNames=('program',),
deprecated=_DEPRECATED,
)


def glInitVertexProgramARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.texture_scale_bias

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_texture_scale_bias'
_DEPRECATED = False
GL_POST_TEXTURE_FILTER_BIAS_SGIX = constant.Constant( 'GL_POST_TEXTURE_FILTER_BIAS_SGIX', 0x8179 )
GL_POST_TEXTURE_FILTER_SCALE_SGIX = constant.Constant( 'GL_POST_TEXTURE_FILTER_SCALE_SGIX', 0x817A )
GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX = constant.Constant( 'GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX', 0x817B )
glget.addGLGetConstant( GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX, (1,) )
GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX = constant.Constant( 'GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX', 0x817C )
glget.addGLGetConstant( GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX, (1,) )


def glInitTextureScaleBiasSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_swizzle

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_swizzle'
_DEPRECATED = False
GL_TEXTURE_SWIZZLE_R_EXT = constant.Constant( 'GL_TEXTURE_SWIZZLE_R_EXT', 0x8E42 )
GL_TEXTURE_SWIZZLE_G_EXT = constant.Constant( 'GL_TEXTURE_SWIZZLE_G_EXT', 0x8E43 )
GL_TEXTURE_SWIZZLE_B_EXT = constant.Constant( 'GL_TEXTURE_SWIZZLE_B_EXT', 0x8E44 )
GL_TEXTURE_SWIZZLE_A_EXT = constant.Constant( 'GL_TEXTURE_SWIZZLE_A_EXT', 0x8E45 )
GL_TEXTURE_SWIZZLE_RGBA_EXT = constant.Constant( 'GL_TEXTURE_SWIZZLE_RGBA_EXT', 0x8E46 )


def glInitTextureSwizzleEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )
